/*
 * Copyright (c) 2007 Luigi Rizzo - Universita` di Pisa
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * $Id: wait.h,v 1.10 2007/01/29 17:34:40 luigi Exp $
 *
 * simple wait/wake_up emulation.
 *
 * We do a very simple emulation, always posting a wakeup, and
 * mapping a wait() into a tsleep. The body of the wait() routine
 * is in the main file as msleep() is #defined to something else here.
 */
#ifndef _LINUX_WAIT_H
#define _LINUX_WAIT_H

#include <linux/init.h>	/* DBG macro */
#include <linux/config.h>
#include <linux/list.h>
#include <linux/stddef.h>
#include <linux/spinlock.h>
#include <asm/system.h>
#include <asm/current.h>

struct __wait_queue_head {
};

typedef struct __wait_queue_head wait_queue_head_t;

static inline void init_waitqueue_head(wait_queue_head_t *q)
{
}

/*
 * Let this always return true. The worst can happen is
 * some extra wakeup()
 */
static inline int waitqueue_active(wait_queue_head_t *q)
{
        //return !list_empty(&q->task_list);
	// DBG("check %p\n", q);
	return 1; // XXX fixme
}

#if 0
void __wake_up(wait_queue_head_t *wq, unsigned int mode,
	int nr_exclusive, void *key)
#endif
#define __wake_up(wq, mode, nr_exclusive, key) do {	\
	if (0) DBG("wakeup %p\n", wq);			\
	wakeup(wq);					\
	} while (0)

#define wake_up_nr(x, nr)		\
	__wake_up(x, TASK_UNINTERRUPTIBLE | TASK_INTERRUPTIBLE, nr, NULL)
#define wake_up(x)	wake_up_nr(x, 1)
#define wake_up_all(x)	wake_up_nr(x, 0)
#define wake_up_interruptible_nr(x, nr)	\
	__wake_up(x, TASK_INTERRUPTIBLE, nr, NULL)
#define wake_up_interruptible(x)	wake_up_interruptible_nr(x, 1)
#define wake_up_interruptible_all(x)	wake_up_interruptible_nr(x, 0)

#if 0
#define wake_up_locked(x)		\
	__wake_up_locked((x), TASK_UNINTERRUPTIBLE | TASK_INTERRUPTIBLE)
#define wake_up_interruptible_sync(x)	\
	__wake_up_sync((x),TASK_INTERRUPTIBLE, 1)
#endif

/**
 * wait_event_interruptible - sleep until a condition gets true
 * @wq: the waitqueue to wait on
 * @condition: a C expression for the event to wait for
 *
 * The process is put to sleep (TASK_INTERRUPTIBLE) until the
 * @condition evaluates to true or a signal is received.
 * The @condition is checked each time the waitqueue @wq is woken up.
 * 
 * wake_up() has to be called after changing any variable that could
 * change the result of the wait condition.
 *
 * The function will return -ERESTARTSYS if it was interrupted by a
 * signal and 0 if @condition evaluated to true.
 */
#define wait_event_interruptible(wq, condition)			\
({								\
	int __ret = 0;						\
	if (0) DBG("wq %p on %s\n", &wq, __stringify(condition));	\
	while (!(condition)) {					\
		__ret = wait_event_body(&wq, 0);		\
		if (__ret == ERESTART) {			\
			__ret = -ERESTARTSYS;			\
			break;					\
		}						\
	}							\
        __ret;							\
})

#endif /* _LINUX_WAIT_H */
